/*
 * ============================================================================
 *
 * Copyright : (c) 2005 SAGEM SA - Site Velizy - URD2
 *
 * The information and source code contained herein is the exclusive
 * property of SAGEM and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the company.
 *
 * ============================================================================
 */

/**
 * \file dsp_util.h DSP util description.
 *
 * This file contain all DSP util description.
 */

#ifndef __DSP_UTIL_H__ /* { */
#define __DSP_UTIL_H__

/*
 * Macros
 * ~~~~~~
 */

/** \name DSP configure Macros.
 *
 * Useful for configure DSP channel.
 *
 * \note
 * Bit operations
 * These operations are used to operaton on the specified bit (or bitfield)
 * within a register.
 *
 * They rely on the following naming conventions:
 *  - XX_BIT            The bit position of the single bit XX
 *  - YY_FST            The bit position of the LSB of bitfield YY
 *  - YY_LEN            The number of bit in the bitfield YY
 *
 * Don't panic too much at the number of operations - the optimizer in the
 * compiler should be able to resolve almost all of them to constants.
 */
/*@{*/
#define DSP_SETBIT(__var__, __bit__, __val__)									\
		if (__val__) {														\
			__var__ |= (1 <<__bit__ ## _BIT );								\
		}																\
		else {															\
			__var__ &= ~(1 <<__bit__ ## _BIT);								\
		}
    
#define DSP_GETBIT(__var__, __bit__)											\
		(((__var__) >> __bit__ ## _BIT) & 0x1)

#define DSP_ZEROMASK(__fld__)													\
		((1 <<(__fld__ ## _LEN)) - 1)
    
#define DSP_FLDMASK(__fld__)													\
		(DSP_ZEROMASK(__fld__) <<(__fld__ ## _FST))
    
#define DSP_SETFIELD(__var__, __fld__, __val__)									\
		(__var__ = (__var__ & ~DSP_FLDMASK(__fld__)) |							\
					(((__val__) & DSP_ZEROMASK(__fld__)) <<__fld__ ## _FST))

#define DSP_GETFIELD(__val__, __fld__)											\
		(((__val__) >> (__fld__ ## _FST)) & DSP_ZEROMASK(__fld__))

/*@}*/

/** \name Compatibility Macros.
 *
 * to keep compatible.
 */
/*@{*/
/** Compatibility to setEncoderSpeechPktSize */
#define setEncoderSpeechPktSize				dsp_setEncoderSpeechPktSize
/** Compatibility to setEncoderAgc */
#define setEncoderAgc						dsp_setEncoderAgc
/** Compatibility to setEncoderDtmf */
#define setEncoderDtmf						dsp_setEncoderDtmf
/** Compatibility to setEncoderSport */
#define setEncoderSport						dsp_setEncoderSport
/** Compatibility to setEncoderType */
#define setEncoderType						dsp_setEncoderType
/** Compatibility to getEncoderType */
#define getEncoderType						dsp_getEncoderType
/** Compatibility to setEncoderActive */
#define setEncoderActive						dsp_setEncoderActive
/** Compatibility to setEncoderG168Ec */
#define setEncoderG168Ec						dsp_setEncoderG168Ec
/** Compatibility to setEncoderVadPktSize */
#define setEncoderVadPktSize					dsp_setEncoderVadPktSize
/** Compatibility to setEncoderVadType */
#define setEncoderVadType					dsp_setEncoderVadType
/** Compatibility to setEncoderG168TailLen */
#define setEncoderG168TailLen					dsp_setEncoderG168TailLen
/** Compatibility to setDecoderSpeechPktSize */
#define setDecoderSpeechPktSize				dsp_setDecoderSpeechPktSize
/** Compatibility to setDecoderToneGen */
#define setDecoderToneGen					dsp_setDecoderToneGen
/** Compatibility to setDecoderPlc */
#define setDecoderPlc						dsp_setDecoderPlc
/** Compatibility to setDecoderSport */
#define setDecoderSport						dsp_setDecoderSport
/** Compatibility to setDecoderType */
#define setDecoderType						dsp_setDecoderType
/** Compatibility to getDecoderType */
#define getDecoderType						dsp_getDecoderType
/** Compatibility to setDecoderActive */
#define setDecoderActive						dsp_setDecoderActive
/** Compatibility to setDecoderVadPktSize */
#define setDecoderVadPktSize					dsp_setDecoderVadPktSize
/** Compatibility to setecoderVadCng */
#define setDecoderVadCng						dsp_setDecoderVadCng
/** Compatibility to setDecoderVarGainCntl */
#define setDecoderVarGainCntl					dsp_setDecoderVarGainCntl
/** Compatibility to setDecoderDtmfRej */
#define setDecoderDtmfRej					dsp_setDecoderDtmfRej
/*@}*/

/*
 * Functions
 * ~~~~~~~~~
 */

/** setEncoderSpeechPktSize.
 *
 *	This function allow to configure Encoder Speech Pkt Size configuration.
 *
 *	@param _voiceOpt	: Voice options.
 *	@param _value		: Value.
 *
 *	@return			: None.
 */
static inline void dsp_setEncoderSpeechPktSize(	VoiceOptions *_voiceOpt,
										unsigned char _value)
{
    DSP_SETFIELD(	_voiceOpt->encoder_opt[0],
				DSP_ENC_SPEECH_PKT_SIZE,
				_value);
}

/** setEncoderAgc.
 *
 * this function allow to set the encoder AGC bit with given value.
 *
 *	@param _voiceOpt	: Voice options.
 *	@param _value		: Value.
 *
 *	@return			: None.
 */
static inline void dsp_setEncoderAgc(			VoiceOptions *_voiceOpt,
										unsigned char _value)
{
    DSP_SETBIT(	_voiceOpt->encoder_opt[0],
				DSP_ENC_AGC_EN,
				_value);
}

/** setEncoderDtmf.
 *
 * this function allow to set the encoder DTMF bit with given value.
 *
 *	@param _voiceOpt	: Voice options.
 *	@param _value		: Value.
 *
 *	@return			: None.
 */
static inline void dsp_setEncoderDtmf(			VoiceOptions *_voiceOpt,
										unsigned char _value)
{
    DSP_SETBIT(	_voiceOpt->encoder_opt[0],
				DSP_ENC_DTMF_EN,
				_value);
}

/** setEncoderSport.
 *
 * This function allow to set the encoder Sport type with given value.
 *
 *	@param _voiceOpt	: Voice options.
 *	@param _value		: Value.
 *
 *	@return			: None.
 */
static inline void dsp_setEncoderSport(			VoiceOptions *_voiceOpt,
										unsigned char _value)
{
    DSP_SETFIELD(	_voiceOpt->encoder_opt[0],
				DSP_ENC_SPORT_TYPE,
				_value);
}

/** getEncoderSport.
 *
 * This function allow to get the given encoder SPORT.
 *
 *	@param _voiceOpt	: Voice options.
 *
 *	@return			: Value.
 */
static inline unsigned char dsp_getEncoderSport(	VoiceOptions *_voiceOpt)
{
    return DSP_GETFIELD(	_voiceOpt->encoder_opt[0],
					DSP_ENC_SPORT_TYPE);
}

/** setEncoderType.
 *
 * This function allow to set the given encoder type.
 *
 *	@param _voiceOpt	: Voice options.
 *	@param _value		: Value.
 *
 *	@return			: None.
 */
static inline void dsp_setEncoderType(			VoiceOptions *_voiceOpt,
										unsigned char _value)
{
    DSP_SETFIELD(	_voiceOpt->encoder_opt[0],
				DSP_ENCODER_TYPE,
				_value);
}

/** getEncoderType.
 *
 * This function allow to get the given encoder type.
 *
 *	@param _voiceOpt	: Voice options.
 *
 *	@return			: Value.
 */
static inline unsigned char dsp_getEncoderType(	VoiceOptions *_voiceOpt)
{
    return DSP_GETFIELD(	_voiceOpt->encoder_opt[0],
					DSP_ENCODER_TYPE);
}

/** setEncoderActive.
 *
 * This function allow to set the encoder active/inactive bit with given value.
 *
 *	@param _voiceOpt	: Voice options.
 *	@param _value		: Value.
 *
 *	@return			: None.
 */
static inline void dsp_setEncoderActive(		VoiceOptions *_voiceOpt,
										unsigned char _value)
{
    DSP_SETBIT(	_voiceOpt->encoder_opt[0],
				DSP_ENCODER_ACTIVE,
				_value);
}

/** setEncoderG168Ec.
 *
 * This function allow to set the encoder G168_EC enable/ disable bit.
 *
 *	@param _voiceOpt	: Voice options.
 *	@param _value		: Value.
 *
 *	@return			: None.
 */
static inline void dsp_setEncoderG168Ec(		VoiceOptions *_voiceOpt,
										unsigned char _value)
{
    DSP_SETBIT(	_voiceOpt->encoder_opt[1],
				DSP_ENC_G168_EC_EN,
				_value);
}

/** setEncoderVadPktSize.
 *
 * This function allow to set the encoder VAD pkt size.
 *
 *	@param _voiceOpt	: Voice options.
 *	@param _value		: Value.
 *
 *	@return			: None.
 */
static inline void dsp_setEncoderVadPktSize(	VoiceOptions *_voiceOpt,
									unsigned char _value)
{
    DSP_SETFIELD(	_voiceOpt->encoder_opt[1],
				DSP_ENC_VAD_PACKET_SIZE,
				_value);
}

/** setEncoderVadType.
 *
 * This function allow to set the encoder VAD type.
 *
 *	@param _voiceOpt	: Voice options.
 *	@param _value		: Value.
 *
 *	@return			: None.
 */
static inline void dsp_setEncoderVadType(		VoiceOptions *_voiceOpt,
										unsigned char _value)
{
    DSP_SETFIELD(	_voiceOpt->encoder_opt[1],
				DSP_ENC_VAD_TYPE,
				_value);
}

/** setEncoderG168TailLen.
 *
 * This function allow to set the encoder G168 tail length.
 *
 *	@param _voiceOpt	: Voice options.
 *	@param _value		: Value.
 *
 *	@return			: None.
 */
static inline void dsp_setEncoderG168TailLen(	VoiceOptions *_voiceOpt,
										unsigned int _value)
{
    DSP_SETFIELD(	_voiceOpt->encoder_opt[2],
				DSP_ENC_G168_TAIL_LENGTH,
				_value);
}

/** setDecoderSpeechPktSize.
 *
 * this function allow to set the decoder speech pkt size.
 *
 *	@param _voiceOpt	: Voice options.
 *	@param _value		: Value.
 *
 *	@return			: None.
 */
static inline void dsp_setDecoderSpeechPktSize(	VoiceOptions *_voiceOpt,
										unsigned char _value)
{
    DSP_SETFIELD(	_voiceOpt->decoder_opt[0],
				DSP_DEC_SPEECH_PKT_SIZE,
				_value);
}

/** setDecoderToneGen.
 *
 * This function allow to set the decoder Tone generation enable/ disable bit.
 *
 *	@param _voiceOpt	: Voice options.
 *	@param _value		: Value.
 *
 *	@return			: None.
 */
static inline void dsp_setDecoderToneGen(		VoiceOptions *_voiceOpt,
										unsigned char _value)
{
    DSP_SETBIT(	_voiceOpt->decoder_opt[0],
				DSP_DEC_TONE_GEN_EN,
				_value);
}

/** setDecoderPlc.
 *
 * This function allow to set the decoder PLC enable/disable bit.
 *
 *	@param _voiceOpt	: Voice options.
 *	@param _value		: Value.
 *
 *	@return			: None.
 */
static inline void dsp_setDecoderPlc(	VoiceOptions *_voiceOpt,
								unsigned char _value)
{
    DSP_SETBIT(	_voiceOpt->decoder_opt[0],
				DSP_DEC_PLC_EN,
				_value);
}

/** setDecoderSport.
 *
 * This function allow to set the decoder Sport type.
 *
 *	@param _voiceOpt	: Voice options.
 *	@param _value		: Value.
 *
 *	@return			: None.
 */
static inline void dsp_setDecoderSport(	VoiceOptions *_voiceOpt,
								unsigned char _value)
{
    DSP_SETFIELD(	_voiceOpt->decoder_opt[0],
				DSP_DEC_SPORT_TYPE,
				_value);
}

/** getDecoderSport.
 *
 * This function alloaw to get the decoder Sport.
 *
 *	@param _voiceOpt	: Voice options.
 *
 *	@return			: value.
 */
static inline unsigned char dsp_getDecoderSport(	VoiceOptions *_voiceOpt)
{
    return DSP_GETFIELD(	_voiceOpt->decoder_opt[0],
					DSP_DEC_SPORT_TYPE);
}

/** setDecoderType.
 *
 * This function allow to set the decoder type.
 *
 *	@param _voiceOpt	: Voice options.
 *	@param _value		: Value.
 *
 *	@return			: None.
 */
static inline void dsp_setDecoderType(	VoiceOptions *_voiceOpt,
								unsigned char _value)
{
    DSP_SETFIELD(	_voiceOpt->decoder_opt[0],
				DSP_DECODER_TYPE,
				_value);
}

/** getDecoderType.
 *
 * This function alloaw to get the decoder type.
 *
 *	@param _voiceOpt	: Voice options.
 *
 *	@return			: value.
 */
static inline unsigned char dsp_getDecoderType(	VoiceOptions *_voiceOpt)
{
    return DSP_GETFIELD(	_voiceOpt->decoder_opt[0],
					DSP_DECODER_TYPE);
}

/** setDecoderActive.
 *
 * This function allow to set the decoder active/inactive bit.
 *
 *	@param _voiceOpt	: Voice options.
 *	@param _value		: Value.
 *
 *	@return			: None.
 */
static inline void dsp_setDecoderActive(	VoiceOptions *_voiceOpt,
									unsigned char _value)
{
    DSP_SETBIT(	_voiceOpt->decoder_opt[0],
				DSP_DECODER_ACTIVE,
				_value);
}

/** setDecoderVadPktSize.
 *
 * This function allow to set the decoder VAD pkt size with the given value.
 *
 *	@param _voiceOpt	: Voice options.
 *	@param _value		: Value.
 *
 *	@return			: None.
 */
static inline void dsp_setDecoderVadPktSize(	VoiceOptions *_voiceOpt,
									unsigned char _value)
{
    DSP_SETFIELD(	_voiceOpt->decoder_opt[1],
				DSP_DEC_VAD_PACKET_SIZE,
				_value);
}

/** setDecoderVadCng.
 *
 * This function allow to set the decoder VAD CNG.
 *
 *	@param _voiceOpt	: Voice options.
 *	@param _value		: Value.
 *
 *	@return			: None.
 */
static inline void dsp_setDecoderVadCng(	VoiceOptions *_voiceOpt,
									unsigned char _value)
{
    DSP_SETFIELD(	_voiceOpt->decoder_opt[1],
				DSP_DEC_VAD_CNG,
				_value);
}


/** setDecoderVarGainCntl.
 *
 * This function allow to set the decoder VAR GAIN Cntl.
 *
 *	@param _voiceOpt	: Voice options.
 *	@param _value		: Value.
 *
 *	@return			: None.
 */
static inline void dsp_setDecoderVarGainCntl(	VoiceOptions *_voiceOpt,
										unsigned char _value)
{
    DSP_SETFIELD(	_voiceOpt->decoder_opt[1],
				DSP_DEC_VAR_GAIN_CNTL,
				_value);
}

/** setDecoderDtmfRej.
 *
 * This function allow to set the encoder DTMF Rejection.
 *
 *	@param _voiceOpt	: Voice options.
 *	@param _value		: Value.
 *
 *	@return			: None.
 */
static inline void dsp_setEncoderDtmfRej(	VoiceOptions *_voiceOpt,
									unsigned char _value)
{
    DSP_SETBIT(	_voiceOpt->encoder_opt[1],
				DSP_ENC_DTMF_REJ_EN,
				_value);
}

/** dsp_getEncoderDtmfRej.
 *
 * This function alloaw to get the dtm reject.
 *
 *	@param _voiceOpt	: Voice options.
 *
 *	@return			: value.
 */
static inline unsigned char dsp_getEncoderDtmfRej(	VoiceOptions *_voiceOpt)
{
    return DSP_GETBIT(		_voiceOpt->encoder_opt[1],
						DSP_ENC_DTMF_REJ_EN);
}

/* dsp_dtmfConv_Char2Dsp.
 *
 *	This function allow to convert Character Dtmf to DSP Dtmf.
 *
 *	@param _charDtmf	: Character Dtmf to convert .
 *
 *	@return			: Dsp Dtmf.
 */
static inline unsigned char dsp_dtmfConv_Char2DspTone(char _charDtmf)
{
	unsigned char _dspDtmf;

	switch(_charDtmf) {
		case '0':
			_dspDtmf	= DIG_0_TONE;
			break;

		case '1':
			_dspDtmf	= DIG_1_TONE;
			break;

		case '2':
			_dspDtmf	= DIG_2_TONE;
			break;

		case '3':
			_dspDtmf	= DIG_3_TONE;
			break;

		case '4':
			_dspDtmf	= DIG_4_TONE;
			break;

		case '5':
			_dspDtmf	= DIG_5_TONE;
			break;

		case '6':
			_dspDtmf	= DIG_6_TONE;
			break;

		case '7':
			_dspDtmf	= DIG_7_TONE;
			break;

		case '8':
			_dspDtmf	= DIG_8_TONE;
			break;

		case '9':
			_dspDtmf	= DIG_9_TONE;
			break;

		case '*':
			_dspDtmf	= DIG_STAR_TONE;
			break;

		case '#':
			_dspDtmf	= DIG_HASH_TONE;
			break;

		case 'A':
			_dspDtmf	= DIG_A_TONE;
			break;

		case 'B':
			_dspDtmf	= DIG_B_TONE;
			break;

		case 'C':
			_dspDtmf	= DIG_C_TONE;
			break;

		case 'D':
			_dspDtmf	= DIG_D_TONE;
			break;

		default:
			return(-1);
			break;
	}

	/* Ok */
	return(_dspDtmf);
}

/* dsp_dtmfConv_dsp2Char.
 *
 *	This function allow to convert Dsp Dtmf To Character Dtmf.
 *
 *	@param _dspDtmf	: Dsp Dtmf to convert .
 *
 *	@return			: Character Dtmf.
 */
static inline char dsp_dtmfConv_DspDigit2Char(unsigned char _dspDtmf)
{
	unsigned char _charDtmf;

	switch(_dspDtmf) {
		case 0:
			_charDtmf='1';
			break;

		case 1:
			_charDtmf='2';
			break;

		case 2:
			_charDtmf='3';
			break;

		case 4:
			_charDtmf='4';
			break;

		case 5:
			_charDtmf='5';
			break;

		case 6:
			_charDtmf='6';
			break;

		case 8:
			_charDtmf='7';
			break;

		case 9:
			_charDtmf='8';
			break;

		case 3:
			_charDtmf='A';
			break;

		case 7:
			_charDtmf='B';
			break;

		case 13:
			_charDtmf='0';
			break;

		case 12:
			_charDtmf='*';
			break;

		case 14:
			_charDtmf='#';
			break;

		case 10:
			_charDtmf='9';
			break;

		default:
			return(-1);
			break;
	}

	/* Ok */
	return(_charDtmf);
}

/* dsp_dtmfConv_Char2DspDigit.
 *
 *	This function allow to convert Character Dtmf to Dsp Dtmf.
 *
 *	@param _charDtmf	: Character Dtmf to convert .
 *
 *	@return			: Dsp Dtmf.
 */
static inline unsigned char dsp_dtmfConv_Char2DspDigit(unsigned	char _charDtmf)
{
	unsigned char _dspDtmf;

	switch(_charDtmf) {
		case '1':
			_dspDtmf = 0;
			break;

		case '2':
			_dspDtmf = 1;
			break;

		case '3':
			_dspDtmf = 2;
			break;

		case '4':
			_dspDtmf = 4;
			break;

		case '5':
			_dspDtmf = 5;
			break;

		case '6':
			_dspDtmf = 6;
			break;

		case '7':
			_dspDtmf = 8;
			break;

		case '8':
			_dspDtmf = 9;
			break;


		case '0':
			_dspDtmf = 13;
			break;

		case '9':
			_dspDtmf = 10;
			break;

		case '*':
			_dspDtmf = 12;
			break;

		case '#':
			_dspDtmf = 14;
			break;

		case 0x80 ://flashhook
			_dspDtmf = 0x80;
			break;


		default:
			return(-1);
			break;
	}

	return(_dspDtmf);
}

#endif /* } */

// vim:ts=5
