#ifndef __PROBE_API_H__
#define __PROBE_API_H__

/******************************************************************************/
/*                       PROBE API DEFINITIONS                                */
/******************************************************************************/

/* Configuration parameter types definition */
/* Definitions for 'ActiveState_t' */
#define PROBE_NOT_ACTIVE 0
#define PROBE_ACTIVE     1
typedef u_int8_t ActiveState_t;

/* Definitions for 'TicketSender_t' */
#define PROBE_NOT_SENDER 0
#define PROBE_SENDER     1
typedef u_int8_t TicketSender_t;

/* Definitions for 'ProbeStatus_s' */
#define SIZE_URL_MAX     (64+1)
#define SIZE_CCODE_MAX   (2+1)
typedef struct ProbeStatus
{
	ActiveState_t probe_state;
	TicketSender_t ticketSend_mode;
	char url_collector1[SIZE_URL_MAX];
	char url_collector2[SIZE_URL_MAX];
	char country_code[SIZE_CCODE_MAX];
	/* Optional URLs & Country code => "-1" if absent */
} ProbeStatus_s;

/* Definitions for 'TicketType_t' */
#define LOG_CALL_QOS     0 /* for call attempt tickets */
#define LOG_SERV_AVAIL   1 /* for service availability tickets */
typedef u_int8_t TicketType_t;

/* Definitions for 'TicketIndex_t' */
#define INDEX_MIN        0
#define INDEX_MAX        499
#define INDEX_ALL        INDEX_MAX
#define INDEX_CYCLIC     INDEX_MAX
typedef u_int16_t TicketIndex_t;


/******************************************************************************/
/*                       PROBE INIT/RAM ACCESS PROTOTYPES DEFINITION          */
/******************************************************************************/

/**
* This function inits (at Init phase and before any other API use) various sizes
* for a proper underlying dimension setting
* <br>=> for RAM storage size checking, 'records*' parameters are used
* <br>=> for cyclic writting detection, 'recordsNb*' parameters are used
* <br>=> for EVENT_RTP_JITTER_BUF sending, 'windowDelay' parameter is used
* <br>=> for EVENT_SERVICE_LOSS after boot, 'delayBefLoss' parameter is used
* @param recordSizeQos The max size (in bytes) of any LOG_CALL_QOS entry
* @param recordsNbQos The max number of entries that LOG_CALL_QOS can store
* @param recordSizeServ The max size (in bytes) of any LOG_SERV_AVAIL entry
* @param recordsNbServ The max number of entries that LOG_SERV_AVAIL can store
* @param windowDelay The window duration (in sec) before EVENT_RTP_JITTER_BUF
* @param delayBefLoss The delay (in min) before EVENT_SERVICE_LOSS after boot
* @return @c 0 on success, @c -1 on errors
*/
extern int voipAPI_init(u_int32_t recordSizeQos,
						TicketIndex_t recordsNbQos,
						u_int32_t recordSizeServ,
						TicketIndex_t recordsNbServ,
						u_int8_t windowDelay,
						u_int8_t delayBefLoss);

/**
* This function updates the Log header related to the given ticket type
* <br>Warning: it may create it first, if not present yet
* @param t_type The tickets Log type
* @param t_header The tickets Log header
* @return @c 0 on success, @c -1 on errors
*/
extern int writeHeader(TicketType_t t_type,
					   const char *t_header);

/**
* This function gets the most recent ticket index within its Log
* @param t_type The tickets Log type
* @param t_index The most recent ticket index
* @return @c 0 on success, @c -1 on errors
*/
extern int getIndexOfLastTicket(TicketType_t t_type,
								TicketIndex_t *t_index);

/**
* This function stores an established Log entry within its Log
* <br>=> t_index = INDEX_CYCLIC : cyclic writting after latest index
* <br>   (oldest entry is erased/replaced, if needed)
* <br>=> t_index < INDEX_CYCLIC : writting at given index
* <br>   (previous entry, if present, gets erased/replaced)
* @param t_type The tickets Log type
* @param t_index The ticket index for storing
* @param t_log_entry The established Log entry
* @return @c 0 on success, @c -1 on errors
*/
extern int writeTicket(TicketType_t t_type,
					   TicketIndex_t t_index,
					   const char *t_log_entry);


/**
* This function extracts a given Log entry from its given Log type
* <br>=> Warning; its associated 't_log_head' is extracted as well
* <br>=> Warning; in case 't_log_entry' is NULL, only 't_log_head' is extracted
* @param t_type The tickets Log type
* @param t_index The ticket index from where the entry is extracted
* @param t_log_head The retrieved Log header
* @param t_log_entry The retrieved Log entry
* @return @c 0 on success, @c -1 on errors
*/
extern int readTicket(TicketType_t t_type,
					  TicketIndex_t t_index,
					  char *t_log_head,
					  char *t_log_entry);


/**
* This function extracts the whole Log contents from its given Log type
* <br>=> t_log_entries allocated by API (client calls deleteLog to free it)
* @param t_type The tickets Log type
* @param n_log_entries The number of existing Log entries within this Log
* @param t_log_head The retrieved Log header
* @param t_log_entries The retrieved Log entries
* @return @c 0 on success, @c -1 on errors
*/
extern int readLog(TicketType_t t_type,
				   TicketIndex_t *n_log_entries,
				   char *t_log_head,
				   char **t_log_entries);

/**
* This function removes the entire specified Log
* @param t_type The tickets Log type to be removed
* @return @c 0 on success, @c -1 on errors
*/
extern int deleteLog(TicketType_t t_type);

/**
* This function allows to read the probe configuration information
* @param probeParams The pointer instance to the retrieved information
* @return @c 0 on success, @c -1 on errors
*/
extern int readAudiphoneConfig(ProbeStatus_s *probeParams);


/******************************************************************************/
/*                       PROBE TICKETS SENDING DEFINITION                     */
/******************************************************************************/

/**
* This function sends an established Log to the VMS
* <br>Warning: this implicitly implies a connection to it first
* <br>Warning: it shall be run within a separate task,
*              if-and-only-if ticketSend_mode is set
* @param url_collector The collector URL
* @param log The established Log
* @return @c 0 on success, @c -1 on errors
*/
extern int sendTicket(const char *url_collector,
					  const char *log);


#endif	/* __PROBE_API_H__ */
