#ifndef __VOIP_API_H__
#define __VOIP_API_H__

/* Actual VoIP API release  (XY stands for vX.Y)      */
#define VOIP_API_VERSION 12 /* 2 digits integer value */

#if defined NEED_UINTS
typedef unsigned char u_int8_t;
typedef unsigned short u_int16_t;
typedef unsigned int u_int32_t;
#endif


/******************************************************************************/
/*                       PROBE API DEFINITIONS                                */
/******************************************************************************/
#include "Probe_API.h"


/******************************************************************************/
/*                       GENERIC VOIP API DEFINITIONS                         */
/******************************************************************************/

/* API registering context identifier definition */
typedef u_int32_t voipCtx_t;

/* Box time type definition */
typedef struct BoxTime
{
	u_int32_t time_sec;
	u_int32_t time_usec;
} BoxTime_s;

/* Box parameter types definition */
#define SIZE_MAC_ADDR   (17+1) /* as per Audiphone spec, with ":" separators  */
#define SIZE_SERIAL_MAX (20+1) /* as per Audiphone spec                       */
#define SIZE_E164_ADDR  (17+1) /* as per Audiphone spec                       */
#define SIZE_STACK_MAX  (5+1)  /* as per Audiphone spec                       */
#define SIZE_MANUF_MAX  (9+1)  /* considering terminal_type of Audiphone spec */
#define SIZE_FIRMV_MAX  (32+1) /* considering terminal_type of Audiphone spec */
#define SIZE_HWV_MAX    (10+1) /* considering terminal_type of Audiphone spec */
#define PROTOCOL_STACK_H323 "Q.931"
#define PROTOCOL_STACK_SIP  "SIP"
typedef struct BoxParam
{
	char serial_nb[SIZE_SERIAL_MAX];
	char mac_adress[SIZE_MAC_ADDR];
	char e164_id[SIZE_E164_ADDR];
	char constructor_name[SIZE_MANUF_MAX];
	char firm_version[SIZE_FIRMV_MAX];
	char hw_version[SIZE_HWV_MAX];
	char protocol[SIZE_STACK_MAX];
} BoxParam_s;

/* Definitions for 'Direction_t' */
#define WAY_INCOMING  1
#define WAY_OUTGOING  2
typedef u_int8_t Direction_t;

/* Definitions for 'VoipEventType_t' 
*  (preferred to enum to control both IDs and type length) */
/* => VoIP service status related */
#define EVENT_SERVICE_GET       0x0001
#define EVENT_SERVICE_LOSS      0x0002
#define EVENT_SERVICE_REJECT    0X0004 /* ARJ (H323),           N/A (SIP)     */
/* => H323 & SIP related events (for SIP : events sequence to be considered)  */
#define EVENT_INITIATING        0x0008 /* SETUP (H323),         INVITE (SIP)  */
#define EVENT_PROCEED           0x0010 /* PROCEEDING (H323),100 TRYING (SIP)  */
#define EVENT_PROGRESS          0x0020 /* PROGRESS (H323),  181,182,183 (SIP) */
#define EVENT_RING              0x0040 /* ALERTING (H323),  180 RINGING (SIP) */
#define EVENT_HANGUP            0x0080 /* CONNECT (H323),   200 OK (SIP)      */
#define EVENT_END               0x0100 /* RELEASE (H323), BYE/RSP-/CANCEL(SIP)*/
#define EVENT_NTP_SYNC          0x0200
/* => Negotiation related events */
#define EVENT_NEGOCIATE_ACK     0x0400 /* OLC_ACK (H323),        Flow (SIP)   */
/* => RTP related events */
#define EVENT_RTP_FIRST_PACKET  0x0800
#define EVENT_RTP_JITTER_BUF    0x1000
#define EVENT_ID_MAX            0x1FFF
#define EVENT_ID_ALL            EVENT_ID_MAX
#define EVENT_ID_TIMEOUT        EVENT_ID_MAX
typedef u_int16_t VoipEventType_t;

/* Definitions for call ending 'Cause_t': as per H.225/Q.931 or SIP standard */
typedef u_int16_t Cause_t;

/* Definitions for 'Codec_t' : as per RFC 3551 standard */
typedef u_int8_t Codec_t;

/* RTP session identifier type definition */
#define SIZE_DOT_NOTATION_ADDR 16
typedef struct SessionID
{
	u_int32_t ssrc;
	char      ip_caller[SIZE_DOT_NOTATION_ADDR];
	char      ip_called[SIZE_DOT_NOTATION_ADDR];
	u_int16_t port_caller;
	u_int16_t port_called;
} SessionID_s;

/* Definitions for terminal type */
#define SIZE_ACCESS_MAX  (9*2+1) /* max[strlen(ACCESS_*)]*2 + 1 */
#define ACCESS_DECT_USB  "DECT USB"
#define ACCESS_FXS       "FXS"
#define ACCESS_SIP       "SIP_PHONE"
#define ACCESS_ISDN      "ISDN"

/* Definitions for any Jitter buffer related data : as per RFC 3550/3551 std  */
typedef u_int32_t Jitter_t;
typedef u_int32_t JitterBufferSize_t;
typedef u_int32_t NetDelay_t;
typedef float LossRate_t;
typedef float Burst_t;
typedef u_int16_t Comm_nb_t;
typedef u_int16_t FrameInfo_nb_t;
#define FRAME_INFO_STATE_OK        0 /* Contiguous RTP frames sequence        */
#define FRAME_INFO_STATE_EMPTY     1 /* Drained buffer frames sequence        */
#define FRAME_INFO_STATE_EMPTY_SID 3 /* (wished) SID frames sequence          */
#define FRAME_INFO_STATE_EMPTY_CN  4 /* (wished) CN frames sequence           */
#define FRAME_INFO_STATE_GAP       5 /* Jitter buffer frame loss sequence     */
typedef u_int32_t FrameInfo_t;
/* The frame information array is built as follows :
 *      array_info := ListOf[(MSB, LSB)] := (FRAME_INFO_STATE, NB_FRAMES)+    */
typedef struct JitterBufferInfo
{
	Jitter_t           jitter_delay;   /* receive inter-arrival jitter in ms  */
	JitterBufferSize_t buffer_size;    /* receive jitter buffer size in ms    */
	NetDelay_t         network_delay;  /* network delay ("0" if undefined)    */
	#if defined NEED_PHASE_1           /* average measurements support only   */
	LossRate_t         loss_rate;      /* receive packet loss rate            */
	Burst_t            lossBurst_size; /* receive loss burst size             */
	#else                              /* full support                        */
	FrameInfo_t        *array_info;    /* receive frame level information     */
	FrameInfo_nb_t     nb_info;        /* number of 'FrameInfo_t' in array    */
	#endif
} JitterBufferInfo_s;

/* Definitions for any VoIP stack related Event */
#define SIZE_CALLID_FIELD 64 /* as per std: 16 for H323, 63 max +'\0' for SIP */
typedef struct VoipEvent
{
	BoxTime_s       timestamp;
	char            call_id[SIZE_CALLID_FIELD];
	VoipEventType_t event_type;
} VoipEvent_s;

/* Definitions for any VoIP service status change Event */
#define LINK_TYPE_INTERNET 0
#define LINK_TYPE_CONVERS  1
typedef u_int8_t VC_used_t;
#define LINK_FAIL_CAUSE_ADSL_DESYNCH  1
#define LINK_FAIL_CAUSE_PPP_LOSS      2
#define LINK_FAIL_CAUSE_UNREGISTER    3
#define LINK_FAIL_CAUSE_VOIPCONF_MISS 4
#define LINK_FAIL_CAUSE_ADMIS_REJECT  5
typedef u_int8_t VoipFailCause_t;
typedef struct VoipStatus  /* VoipEvent_s shall remain located at first field */
{
	VoipEvent_s voip_event; /* for VoipService[Get|Fail]: event_type use only */
	VC_used_t   vc_used;    /* for both VoipService[Get|Fail]                 */
} VoipStatus_s;


/******************************************************************************/
/*                       VOIP EVENTs DEFINITION                               */
/******************************************************************************/
/*                                                                            */
/* Note : these VOIP EVENTs are ALL made with :                               */
/* ----   => 'VoipEvent_s' head instance (possibly a 'VoipStatus_s' one)      */
/*        => (optional)    event type specific contents                       */
/*                                                                            */
/******************************************************************************/

/* VoipEventInitiating => head.event_type := EVENT_INITIATING                 */
struct VoipEventInitiating
{
	VoipEvent_s head;         /* Fixed event head, conveying 'event_type'     */
	Direction_t init_way;     /* Way of call establishment                    */
	char        e164_caller[SIZE_E164_ADDR]; /* E164 caller alias             */
	char        e164_called[SIZE_E164_ADDR]; /* E164 called alias             */
};


/* VoipEventProceeding => head.event_type := EVENT_[PROCEED|PROGRESS|RING]    */
struct VoipEventProceeding 
{
	VoipEvent_s head;         /* Fixed event head, conveying 'event_type'     */
};

/* VoipEventHangup => head.event_type := EVENT_HANGUP                         */
/*                 => Warning: support for FACILITY need to be checked        */
struct VoipEventHangup
{
	VoipEvent_s head;         /* Fixed event head, conveying 'event_type'     */
};

/* VoipEventEnding => head.event_type := EVENT_END                            */
struct VoipEventEnding
{
	VoipEvent_s head;         /* Fixed event head, conveying 'event_type'     */
	Direction_t end_way;      /* Way of call ending                           */
	Cause_t     cause;        /* Standard cause code for call termination     */
	LossRate_t  net_loss_rate;/* Network pkt loss rate (-1 if not connected)  */
	Comm_nb_t   nbComm_mean;  /* Mean number of simultaneous communications   */
	Comm_nb_t   nbComm_max;   /* Max. number of simultaneous communications   */
};

/* VoipEventSetupComplete => head.event_type := EVENT_NEGOCIATE_ACK           */
/*   => with 'terminal_used' := <terminal_name>[-<interface_name>]            */
/*   => [-<interface_name>] part being provided optionnaly (for SIP phones)   */
struct VoipEventSetupComplete
{
	VoipEvent_s head;         /* Fixed event head, conveying 'event_type'     */
	SessionID_s session;      /* Ssrc field -not used- at this stage          */
	char        terminal_used[SIZE_ACCESS_MAX]; /* Used access to communicate */
};

/* VoipEventFirstPacket => head.event_type := EVENT_RTP_FIRST_PACKET          */
/*   => Warning: needed in both ways on effective voice packet                */
/*               (i.e.: neither an SID nor a ComfortNoise packet              */
/*   => Warning: might be received again next to a VC change                  */
/*               (in case the communication remains active on other VC)       */
struct VoipEventFirstPacket
{
	VoipStatus_s head;        /* Event head, conveying 'event_type' & 'VC'    */
	SessionID_s  rtp_id;      /* Associated with head.call_id                 */
	Direction_t  flow_way;    /* Way of RTP flow                              */
	Codec_t      payloadType; /* Conveyed codec                               */
	u_int32_t    payloadSize; /* Bytes => payloadSize (sec) & nbFrames_perPkt */
};

/* VoipEventInfoPackets => head.event_type := EVENT_RTP_JITTER_BUF            */
/*   => Warning: received at least every "WindowDelay" sec                    */
/*   => Warning: also received on last communication segment (i.e. call end)  */
/*   => Warning: not received if NO incoming RTP data (i.e. all frames lost)  */
struct VoipEventInfoPackets
{
	VoipEvent_s        head;  /* Fixed event head, conveying 'event_type'     */
	SessionID_s        rtp_id;/* Associated with head.call_id                 */
	JitterBufferInfo_s info;  /* Jitter buffer information update             */
};

/* VoipEventNTPSync => head.event_type := EVENT_NTP_SYNC                      */
#define TIME_REF_BEFORE_THEN  0 /* new times are before then */
#define TIME_REF_AFTER_THEN   1 /* new times are after  then */
typedef u_int8_t TimeStatus_t;
struct VoipEventNTPSync
{
	VoipEvent_s head;      /* Fixed event head, conveying 'event_type'        */
	BoxTime_s   ntp_delta; /* Time offset vs previous time reference          */
	TimeStatus_t status;   /* Synchronization status                          */
};

/* VoipServiceFail => head.event_type := EVENT_SERVICE_[LOSS|REJECT]          */
struct VoipServiceFail
{
	VoipStatus_s    head;     /* Fixed event head, conveying 'event_type'     */
	VoipFailCause_t cause;    /* VoIP service failure cause                   */
};

/* VoipServiceGet => head.event_type := EVENT_SERVICE_GET                     */
#define SIZE_NAME_MAX 64  /* as per Audiphone spec                            */
#define SIZE_GKID_MAX 32  /* as per Audiphone spec                            */
struct VoipServiceGet
{
	VoipStatus_s head;        /* Fixed event head, conveying 'event_type'     */
	char         ip_addr[SIZE_DOT_NOTATION_ADDR]; /* client IP address        */
	char         bas_id[SIZE_NAME_MAX]; /* BAS name                           */
	char         ecr_id[SIZE_NAME_MAX]; /* ECR name ("-1" if unknown)         */
	char         pfs_id[SIZE_GKID_MAX]; /* Gatekeeper (H.323) or SBC (SIP) id */
};


/******************************************************************************/
/*                       VOIP EVENTS PROTOTYPES DEFINITION                    */
/******************************************************************************/

/**
* This function is returning by the VoIP stack :
* <br>=> whenever a VoIP event occurs (i.e. 'VoipEvent_s' conveyed)
* <br>=> whenever a VoIP serv status change occurs (i.e. 'VoipStatus_s' conveyed)
* @param voipCtx_ptr The registered context identifier
* @param voipEvent_instance The pointer instance to the received VoIP event
* @param timeout The time interval to wait for prior to returning
* <br>=> either the client or the provider (upon success) may then modify it
* <br>=> if NULL, this call blocks indefinitely until an EVENT gets received
* <br>=> if not, at expiration, voipEvent_instance may point to EVENT_ID_TIMEOUT
* <br>   (specifying the call_id, still active, on which an exception has
* <br>    occurred, e.g. : internal stack error)
* @return @c 0 on success, @c -1 on errors
*/
extern int onVoipEvent(voipCtx_t voipCtx_ptr,
					   const void *voipEvent_instance,
					   BoxTime_s *timeout);


/******************************************************************************/
/*                       VOIP INIT/GET PROTOTYPES DEFINITION                  */
/******************************************************************************/

/**
* This function allows the client application to register itself
* to the VOIP Stack API
* @param clientName The client name for registering, made as follows:
* <br>=> VOIP_API_VERSION[_NAME]
* @param voipCtx_ptr The pointer instance to the registered context identifier
* <br>=> first, VOIP_API_VERSION is checked by the API provider
* <br>=> upon success, a VERSIONzt positive identifier is provided as follows:
* <br>=>   - VERSION := VOIP_API_VERSION or compatible supported version
* <br>=>   - zt      := {0...99}
* @return @c 0 on success, @c -1 on errors
*/
extern int voipAPI_register(const char *clientName,
							voipCtx_t *voipCtx_ptr);

/**
* This function subscribes to a given VoIP event :
* <br>=> conveying either a 'VoipEvent_s' or a 'VoipStatus_s' instance
* @param voipCtx_ptr The registered context identifier
* @param voipEvent The VoIP event (EVENT_ID_ALL for interest in all events)
* <br>=> never subscribe to EVENT_ID_TIMEOUT, implicitly done via onVoipEvent
* @return @c 0 on success, @c -1 on errors
*/
extern int subscribe(voipCtx_t voipCtx_ptr,
					 VoipEventType_t voipEvent);

/**
* This function clears a given VoIP event subscription :
* @param voipCtx_ptr The registered context identifier
* @param voipEvent The VoIP event (EVENT_ID_ALL for clearing)
* @return @c 0 on success, @c -1 on errors
*/
extern int unsubscribe(voipCtx_t voipCtx_ptr,
					   VoipEventType_t voipEvent);

/**
* This function allows the client application to unregister itself
* from the VOIP Stack API
* @param voipCtx_ptr The registered context identifier
* @return @c 0 on success, @c -1 on errors
*/
extern int voipAPI_unregister(voipCtx_t voipCtx_ptr);

/**
* This function allows to read the system box information
* <br>Warning: if not yet available, 'e164_id' is set to "-1" (not an error case)
* @param boxParams The pointer instance to the current system box information
* @return @c 0 on success, @c -1 on errors
*/
extern int readBoxParameters(BoxParam_s *boxParams);

/**
* This function allows to get the current box time since Epoch
* @param time The pointer instance to the current box time
* @return @c 0 on success, @c -1 on errors
*/
extern int getBoxTimeOfDay(BoxTime_s *time);


#endif	/* __VOIP_API_H__ */
